#ifndef GLOBAL_H_
#define GLOBAL_H_

#include "common.h"
#include "gogo.h"
#include "l3side.h"
#include "thread.h"

#ifdef _GOGO_C_
	#define EXT
#else
	#define EXT extern
#endif

enum vbr_mode_e {
  vbr_off=0,
  vbr_rh,
  vbr_abr,
  vbr_default=vbr_rh  /* change this to change the default VBR mode of LAME */ 
};


/**
 *  ATH related stuff, if something new ATH related has to be added,
 *  please plugg it here into the ATH_t struct
 */
typedef struct ath_t
{
    int     use_adjust;     // do we want to use the auto adjustment yes/no
    FLOAT8  adjust;         // lowering based on peak volume, 1 = no lowering
    FLOAT8  adjust_limit;   // limit for dynamic ATH adjust
    FLOAT8  decay;          // determined to lower x dB each second
    FLOAT8  l[SBMAX_l];     // ATH for sfbs in long blocks
    FLOAT8  s[SBMAX_s];     // ATH for sfbs in short blocks
    FLOAT8  cb[CBANDS];     // ATH for convolution bands
} ATH_t;


/**
 * "bit_stream.h" Type Definitions 
 */
typedef struct  bit_stream_struc {
    unsigned char *buf;         /* bit stream buffer */
    int         buf_size;       /* size of buffer (in number of bytes) */
    int         totbit;         /* bit counter of bit stream */
    int         buf_byte_idx;   /* pointer to top byte in buffer */
    int         buf_bit_idx;    /* pointer to top bit of top byte in buffer */
    
    /* format of file in rd mode (BINARY/ASCII) */
} Bit_stream_struc;

/**
 *  VBR_seek_info_t
 */
typedef struct vbr_seek_info_t
{
    int sum;    // what we have seen so far
    int seen;   // how many frames we have seen in this chunk
    int want;   // how many frames we want to collect into one chunk
    int pos;    // actual position in our bag
    int size;   // size of our bag
    int bag[400];   // bag
} VBR_seek_info_t;

#define MAX_HEADER_BUF 256
#define MAX_HEADER_LEN 40 /* max size of header is 38 */
/**
 * variables for bitstream.c 
 * mpeg1: buffer=511 bytes  smallest frame: 96-38(sideinfo)=58
 * max number of frames in reservoir:  8 
 * mpeg2: buffer=255 bytes.  smallest frame: 24-23bytes=1
 * with VBR, if you are encoding all silence, it is possible to
 * have 8kbs/24khz frames with 1byte of data each, which means we need
 * to buffer up to 255 headers!
 * also, max_header_buf has to be a power of two */
typedef struct header_buf_t {
    int write_timing;
    int ptr;
    char buf[MAX_HEADER_LEN];
} Header_buf_t;

typedef struct abr_t {
	float a;
	float b;
	float r;
} Abr_t;

typedef enum {
	IDEV_ILL,
	IDEV_FILE,
	IDEV_UFUNC,
	IDEV_STDIN
} GOGO_INPDEV;

typedef enum {
	ODEV_ILL,
	ODEV_FILE,
	ODEV_UFUNC,
	ODEV_STDOUT
} GOGO_OUTDEV;

typedef enum {
	IFMT_ILL,
	IFMT_RAW,
	IFMT_WAV,
	IFMT_AIF
} GOGO_INPFMT;

typedef enum {
	OFMT_ILL,
	OFMT_MP3,
	OFMT_WAV,
	OFMT_RMP
} GOGO_OUTFMT;

/**
 *  riffInfo_t
 */
typedef struct riffInfo_t 
{
	int		riffInfosLen;
	char	*pRiffInfos;
} RiffInfo_t;


/**
 * the following variables are constant(Read Only) in encoding 
 * Don't change them except in gogo.c 
 * ̕ϐ̓GR[h͒萔ł 
 * gogo.cȊOł͏ł͂Ȃ 
 */
typedef struct ro_t {
/*  align KvȃoB */
/*  i386 Ŋeo size  16 ̔{ɂȂ悤ɂ邱 */
  /* fft.c    */
  FLOAT window[BLKSIZE];  // 4096 bytes
  FLOAT window_s[BLKSIZE_s/2]; // 512 bytes 

  /* DATA FROM PSYMODEL.C */
/* The static variables "r", "phi_sav", "new", "old" and "oldest" have    */
/* to be remembered for the unpredictability measure.  For "r" and        */
/* "phi_sav", the first index from the left is the channel select and     */
/* the second index is the "age" of the data.                             */
  FLOAT8	minval[CBANDS];                    // 256 bytes
  FLOAT8 s3_s[CBANDS][CBANDS];                     // 16384 bytes
  FLOAT8 s3_l[CBANDS][CBANDS];                     // 16384 bytes

  /* Scale Factor Bands    */
//  FLOAT8	w1_l[SBMAX_l], w2_l[SBMAX_l]; // RO
//  FLOAT8	w1_s[SBMAX_s], w2_s[SBMAX_s]; // RO /* always 0.5 */
  int	s3ind[CBANDS][2];  // 512 bytes
  int	s3ind_s[CBANDS][2]; // 512 bytes
  FLOAT8 SNR_s[CBANDS]; // 256 bytes

  int	numlines_s[CBANDS]; // 256 bytes
  int	numlines_l[CBANDS]; // 256 bytes

  FLOAT8 mld_l[SBMAX_l+2];  // 88+ 8 bytes
  FLOAT8 mld_s[SBMAX_s+3];  // 52+12 bytes
  int	 bu_l[SBMAX_l+2];   // 88+ 8 bytes 
  int    bo_l[SBMAX_l+2];   // 88+ 8 bytes
  int	 bu_s[SBMAX_s+3];   // 52+12 bytes
  int    bo_s[SBMAX_s+3];   // 52+12 bytes
/*  align KvȃoBI */

/*	ftHgݒȂGR[hɎgp郁oB */
	/* internal variables */
	int nSample;				/* Tvs 0 or ƂĂ */
	int framesize;
	int	nChannel;
	int out_samplerate;         /* output_samp_rate.
                                   default: LAME picks best value 
                                   at least not used for MP3 decoding:
                                   Remember 44.1 kHz MP3s and AC97           */
	int	channels_out;	/* number of channels in the output data stream */ // RO
	int	mode_gr;

	/* CPU */
	int nCPU;

	/* Thread */
	int	nThread;
	gogo_thread_data	*tl;

	/* VBR control */
    int VBR;

	/* etc */
	int debug;
	int silent;
	void (*printf)(const char *, ...);

	/* used for padding */
	int frac_SpF; 

	/* from quantize_pvt.h */
	#define IXMAX_VAL 8206  /* ix always <= 8191+15.    see count_bits() */
	/* buggy Winamp decoder cannot handle values > 8191 */
	/* #define IXMAX_VAL 8191 */
	#define Q_MAX 330
	#define PRECALC_SIZE (IXMAX_VAL+2)	
	float	pow20[Q_MAX];
	float	ipow20[Q_MAX];
	float	pow43[PRECALC_SIZE];

	/* variables used by quantize.c */
	FLOAT8 decay; 

	int sideinfo_len;

	scalefac_struct scalefac_band; 

	int cw_upper_index; 
	int	npart_l,npart_s; 
	int	npart_l_orig,npart_s_orig;
	int npart_l_pre_max;

	int	emphasis;
	int samplerate_index;  

	/* file */
	struct MCP_INPDEV_USERFUNC userIn;	/* if user.pUserFunc == NULL then using file I/O */
	MPGE_USERFUNC userOut;
/*	ftHgݒȂGR[hɎgp郁oBI */

/*	ASM ֐ւ̃|C^B */
#ifdef CPU_I386
#endif // CPU_I386
/*	ASM ֐ւ̃|C^BI */

/*	tOIoB */
	/* internal variables */
	unsigned char	quality;	/* 2:slow, 5:default, 7:fast */
	unsigned char	mode;		/* mode = STEREO, JOINT_STEREO, MONO */
	unsigned char	version;                    /* 0=MPEG-2  1=MPEG-1  (2=MPEG-2.5)     */
    unsigned char	lame_encode_frame_init;     
	unsigned char	noise_shaping;      /* 0 = none 1 = ISO AAC model (2 = allow scalefac_select=1) */
	unsigned char	noise_shaping_amp;	/*  0 = ISO model: amplify all distorted bands
											1 = amplify only most distorted band
											2 = amplify bands using? 
											3 = amplify bands using? */ 
	unsigned char	noise_shaping_stop;	/*	0 = stop at over=0, all scalefacs amplified or
												a scalefac has reached max value
											1 = stop when all scalefacs amplified or        
												a scalefac has reached max value
											2 = stop when all scalefacs amplified  */
	unsigned char	use_best_huffman;	/* 0 = no. 1=outside loop */
	unsigned char	use_filtering;		/* 0 = no. 1=filtering(lowpass or highpass or both) */
	unsigned char	dummybyte1;
	unsigned char	dummybyte2;
	unsigned char	dummybyte3;
/*	tOIoBI */

/*	-v  -q IvVɂĂ̓GR[hɎgp郁oB */
	/*
	 * set either brate>0  or compression_ratio>0, LAME will compute
     * the value of the variable not set.
     * Default is compression_ratio = 11.025
     */
	int brate;              /* bitrate */

	/* VBR control */
    int VBR_q;
    int VBR_min_bitrate_kbps;
    int VBR_max_bitrate_kbps;

	int VBR_min_bitrate;            /* min bitrate index */ // RO
	int VBR_max_bitrate;            /* max bitrate index */ // RO

	/* file */
	int useVbrTagWithUserOut;
/*	-v  -q IvVɂĂ̓GR[hɎgp郁oBI */

/*	̑ݒɂĂ̓GR[hɎgp郁oB */
	// int fill_buffer_resample_init; // ݖgp resampling ɕ邩
//	int encFreqHz;
//	int outFreqHz;

  /* polyphase filter */
  FLOAT8 amp_lowpass[32]; // 128 bytes
  FLOAT8 amp_highpass[32]; // 128 bytes
  int lowpass_band;          /* zero bands >= lowpass_band in the polyphase filterbank */  // RO
  int highpass_band;         /* zero bands <= highpass_band */  // RO
  int lowpass_start_band;    /* amplify bands between start */  // RO
  int lowpass_end_band;      /* and end for lowpass */ // RO
  int highpass_start_band;   /* amplify bands between start */ // RO
  int highpass_end_band;     /* and end for highpass */ // RO
/*	̑ݒɂĂ̓GR[hɎgp郁oBI */

/*	GR[hɂ͎gȂoB
	Ft@C(GR[hJn̂)
		VBR^O֘A(GR[hÎ)
*/
	int lastFrameSamples;

	int inpFreqHz;
	int rateKbps;
	int CBR_bitrate;

	/* CPU */
	int unit;

	/* file */
	char inFileName[MAX_FILE_LEN];
	char outFileName[MAX_FILE_LEN];

	/* VBR tag */
	int nZeroStreamSize;
	int TotalFrameSize;

	/* RIFF RMP */
	RiffInfo_t riffInfo;

	GOGO_INPDEV input_device;			/* input device FILE/USERFUNC/STDIN */
	GOGO_INPFMT input_format;			/* input file format RAW/WAVE/AIFF */
	GOGO_OUTDEV output_device;			/* output device FILE/USERFUNC/STDOUT */
	GOGO_OUTFMT output_format;			/* output file format MP3/WAVE/RIFF */

	// ͕̂KvȂ́H --> rbg[gɉătB^̃JbgItlݒ肷Kv̂ŗv 
    /* resampling and filtering */
    int lowpassfreq;        /* freq in Hz. 0=gogo choses.  -1=no filter  */
    int highpassfreq;       /* freq in Hz. 0=gogo choses.  -1=no filter  */
    int lowpasswidth;       /* freq width of filter, in Hz (default=15%) */
    int highpasswidth;      /* freq width of filter, in Hz (default=15%) */

	/* lowpass and highpass filter control */
	float lowpass1,lowpass2;        /* normalized frequency bounds of passband */  // RO
	float highpass1,highpass2;      /* normalized frequency bounds of passband */  // RO

	/*	GR[hɂ͎gȂoBI */
} RO_t;

/**
 * ̕ϐ̓GR[h͕ϐł 
 * }`XbhGR[ĥƂ͋Lɒuׂϐł 
 * ACgLbVɒ 
 */
typedef struct rw_t {
	/* variables for newmdct.c */
	float	subband_buf[2][18][SBLIMIT];	/* PFB -> MDCT(shared part) */
	short	sample_buf[2*2*1152];	/* sample input -> shiftin(shared part) */
	/* psymodel.c */
	FLOAT	cw[HBLKSIZE*4]; // shared rOK
	III_psy_xmin	thm[4]; // shared rOK
	III_psy_xmin	en[4];  // shared rOK
	float	tot_ener[4];	// shared
	/* ratios  */
	float	pe[4]; // shared rOK
	float	nb_12[4][CBANDS*2];/* 2i=>nb_1, 2i+1=>nb_2 */	// shared rOK

	short	*fr0, *fr1, *fr2, *fr3; // shared

  /*  
   * Some remarks to the Class_ID field:
   * The Class ID is an Identifier for a pointer to this struct.
   * It is very unlikely that a pointer to lame_global_flags has the same 32 bits 
   * in it's structure (large and other special properties, for instance prime).
   *
   * To test that the structure is right and initialized, use:
   *     if ( gfc -> Class_ID == LAME_ID ) ...
   * Other remark:
   *     If you set a flag to 0 for uninit data and 1 for init data, the right test
   *     should be "if (flag == 1)" and NOT "if (flag)". Unintended modification
   *     of this element will be otherwise misinterpreted as an init.
   */
  
	unsigned int	num_samples_read; // shared
	int		num_samples_to_encode;  // shared

	int frameNum;                   /* number of frames encoded             */ // shared
	int totalframes; // shared

	/* psycho acoustics */
  FLOAT8 ms_ratio[2];  // shared
  /* used for padding */
  int slot_lag; // shared(modefied in lame_encode_mp3_frame()) rOK

  /* variables used by quantize.c */
  int OldValue[2]; // shared	quantize.c
  int CurrentStep; // shared	quantize.c

  char bv_scf[576]; // shared	takehiro.c
  
  int sfb21_extra; /* will be set in lame_init_params */  // shared	quantize.c, quantize_pvt.c
  
  Header_buf_t header[MAX_HEADER_BUF]; // shared

  int h_ptr; // shared
  int w_ptr; // shared
  int ancillary_flag; // shared
  

// ܂ŔrOK
/* ߂߂
	ResvSize }Y
*/

	Bit_stream_struc   bs;  // shared rOK
  int ResvSize; /* in bits */ // shared	ResvFrameBegin(), ResvMaxBits()ŎQƁAResvAdjust(), ResvFrameEnd(), bitstream.cŕύX
  int ResvMax;  /* in bits */ // shared rOK(mutex_mp3out, mutex_l3encŔr邱)
	int	main_data_begin;	// shared rOK(mutex_mp3out, mutex_l3encŔr邱)

  /* DATA FROM PSYMODEL.C */
/* The static variables "r", "phi_sav", "new", "old" and "oldest" have    */
/* to be remembered for the unpredictability measure.  For "r" and        */
/* "phi_sav", the first index from the left is the channel select and     */
/* the second index is the "age" of the data.                             */

  
  /* unpredictability calculation
   */
  Abr_t sav[2][6][4]; /* 6 = cw_lower_idx */ // shared rOK


  FLOAT8 ms_ratio_s_old,ms_ratio_l_old; // shared rOK

  /* block type */
  int	blocktype_old[2]; // shared rOK(psysse.nasintƂĎgĂ)

  ATH_t	ATH;   // all ATH related stuff // RO VBRRW`

  /* VBR tag */
  VBR_seek_info_t VBR_seek_table; 

#ifdef	MT_ENCODER
	/* mutex */
#define	MAX_CRITICAL_REGION	9
	gogo_mutex	mutex[MAX_CRITICAL_REGION];

#define SEMAPHORE_STARTENCODE	0
#define SEMAPHORE_GETPCM_IN		1
#define SEMAPHORE_GETPCM_OUT	2
#define MAX_SEMAPHORE			3
	gogo_semaphore semaphore[MAX_SEMAPHORE];

	gogo_thread_data *end_tl;
	int trackencode_flag;
	int termencode_flag;
	int getpcm_status;
#endif
} RW_t;

#ifdef CPU_I386
// i386 ł vars.nas Œ`
extern RO_t RO;
extern RW_t RW;
#else
// i386@ȊOł main.c Œ`
EXT RO_t RO;
EXT RW_t RW;
#endif

#define IS_MPEG1		(RO.version)

/* mark as original, always 1            */
#define ORIGINAL		(1)

/* mark as copyright, always 0           */
#define COPYRIGHT		(0)

/* 0=no padding, 1=always pad,
   2=adjust padding, always 2            */
#define PADDING_TYPE	(2)

/* uses:
	*15bit SIMD instructions
	*23bit fdiv
*/
#define	USE_LOW_PRECISIOIN	(9 <= RO.quality)

#endif /* GLOBAL_H_ */
